package cn.iocoder.yudao.module.customer.controller.admin.customer;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.complaint.CustomerComplaintDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerFollowup.CustomerFollowupDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customer.commission.CustomerCommissionService;
import cn.iocoder.yudao.module.customer.service.customer.complaint.CustomerComplaintService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.service.customerFollowup.CustomerFollowupService;
import cn.iocoder.yudao.module.customer.service.customerLevelLog.CustomerLevelLogService;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.CustomerOperateLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.order.dal.dataobject.targetLog.TargetLogDO;
import cn.iocoder.yudao.module.order.service.targetLog.TargetLogService;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrandempower.ProductBrandEmpowerDO;
import cn.iocoder.yudao.module.product.service.productbrandempower.ProductBrandEmpowerServiceImpl;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayment.CommissionPaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.service.commissionPayable.CommissionPayableService;
import cn.iocoder.yudao.module.wealth.service.commissionPayment.CommissionPaymentService;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptServiceImpl;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Log4j2
@Service
@Validated
public class CustomerMergeServiceImpl implements CustomerMergeService {




    @Resource
    private OrderApi orderApi;


    @Resource
    OfferService offerService;

    //客户跟进
    @Resource
    CustomerFollowupService customerFollowupService;

    @Resource
    private CustomerComplaintService customerComplaintService;


    @Resource
    private ReceiptServiceImpl receiptService;

    @Resource
    private ProductBrandEmpowerServiceImpl productBrandEmpowerService;

    //客户登记日志
    @Resource
    private CustomerLevelLogService customerLevelLogService;

    @Resource
    private CustomerOperateLogService customerOperateLogService;

    @Resource
    private TargetLogService targetLogService;

    @Resource
    private CommissionPayableService commissionPayableService;

    @Resource
    private CommissionPaymentService commissionPaymentService;

    @Resource
    private CustomerContactsService customerContactsService;

    @Resource
    private CustomerCommissionService customerCommissionService;

    @Resource
    private CustomerService customerService;



    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean doMergeCus(CustomerDO customerDOSaved, CustomerDO customerDODeleted, long loginUserId) {

        /*
        提交验证：
        当保留客户和非主客户的客户经理均不为空且不是同一个用户(允许客户在公海)，
        弹出提示：保留客户和非主客户，不能在两个不同的客户经理名下，
        请先确认两个客户在同一个客户经理名下，不允许提交
        当非主客户/保留客户在移交过程中，不允许合并，弹出提示
        当允许提交，先弹出上面的注意事项提示，确认后提交
        1. 非主客户中的客户联系人所有信息追加到保留客户中，注意：关联账号也需要一并追加
        2. 非主客户中的所有关联订单中的客户编号需要更新为保留客户，业绩归属为非主客户，
        也需要同时更新为保留客户（此处需要注意有一些订单的收发货人是同一个客户，
        还需要注意控货订单的情况，控货权）
        3. 非主客户的跟进记录，报价单，客户投诉，品牌授权，
        财务相关单据（收款单，佣金请款单，佣金付款单），都需要更新为保留客户
        4. 非主客户关联的注册会员信息
        4. 提交成功后，非主客户删除，插入记录到保留客户，
        备注：客户合并，删除XXX非主客户编号
         注意：此处要求是在保留客户中能查询到非主客户的所有信息，
         不清楚数据库逻辑结构，此处需要开发查找出所有的关联关系
         非主客户关联的订单/收款单/佣金情况单，佣金付款单有在途流程，
         需要测试这种情况是否影响合并
         */


        //合并订单  在合并订单前会验证非主客户是否有审批中的订单，若存在则不能合并抛出异常
        orderApi.mergeOrder(customerDOSaved.getId(), customerDODeleted.getId());

        //把客户2的联系方式合并到客户1  1
        doMogeContact(customerDOSaved, customerDODeleted, loginUserId);
        //合并报价单  2
        doMogeOffer(customerDOSaved, customerDODeleted, loginUserId);
        //合并跟进记录  3
        doMergeCustomerFollowup(customerDOSaved, customerDODeleted, loginUserId);
        //合并客户投诉  4
        doMergeCustomerComplaint(customerDOSaved, customerDODeleted, loginUserId);
        //合并产品品牌授权  6
        doMergeProductBrandEmpower(customerDOSaved,customerDODeleted,loginUserId);
        //合并客户登记日志  7
        doMergeCustomerLevelLog(customerDOSaved,customerDODeleted,loginUserId);
        //合并信用日志  8
        customerService.doMergeCustomerCreditLog(customerDOSaved.getId(),
                customerDODeleted.getId());

        //合并操作日志  9
        // customerService.doMergeCustomerOperateLog(customerDO1.getId(),
        //         customerDODeleted.getId());

        //合并收款单信息  5
        doMergeeReceipt(customerDOSaved, customerDODeleted, loginUserId);
        // 合并订单佣金应付款
        doMergeComissionPayable(customerDOSaved, customerDODeleted);
        // 合并佣金付款单
        doMergeCommissionPayment(customerDOSaved, customerDODeleted);
        // 合并业绩
        doMergeTargetLog(customerDOSaved, customerDODeleted);

        // 删除客户
        customerService.deleteCustomer(customerDODeleted.getId());

        // 纪录客户合并日志
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                .setOperator(loginUser == null ? null : loginUser.getId())
                .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                .setCustomerId(customerDOSaved.getId())
                .setNumber(customerDOSaved.getNumber())
                .setName(customerDOSaved.getName())
                .setOldCustomerService(null)
                .setNewCustomerService(customerDOSaved.getCustomerService())
                .setOldEstimateEnterOpenSeaTime(null)
                .setNewEstimateEnterOpenSeaTime(customerDOSaved.getEstimateEnterOpenSeaTime())
                .setOrderId(null)
                .setOrderNo(null)
                .setOperateType(CustomerOperateTypeEnum.CUSTOMER_MERGE.getValue())
                .setRemark(String.format("客户合并，删除非主客户【%d-%s-%s】", customerDODeleted.getId(), customerDODeleted.getNumber(), customerDODeleted.getName()));
        customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);

        //纪录被删除的客户的日志

        CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO2 = new CustomerOperateLogCreateReqVO()
                .setOperator(loginUser == null ? null : loginUser.getId())
                .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                .setCustomerId(customerDODeleted.getId())
                .setNumber(customerDODeleted.getNumber())
                .setName(customerDODeleted.getName())
                .setOldCustomerService(null)
                .setNewCustomerService(customerDODeleted.getCustomerService())
                .setOldEstimateEnterOpenSeaTime(null)
                .setNewEstimateEnterOpenSeaTime(customerDODeleted.getEstimateEnterOpenSeaTime())
                .setOrderId(null)
                .setOrderNo(null)
                .setOperateType(CustomerOperateTypeEnum.CUSTOMER_MERGE.getValue())
                .setRemark(String.format("客户合并，将此客户合并到主客户【%d-%s-%s】", customerDOSaved.getId(), customerDOSaved.getNumber(), customerDOSaved.getName()));
        customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO2);


        return true;
    }





    private void doMergeCommissionPayment(CustomerDO customerDOSaved, CustomerDO customerDODeleted) {
        commissionPaymentService.update(new LambdaUpdateWrapper<CommissionPaymentDO>()
                .set(CommissionPaymentDO::getCustomerId, customerDOSaved.getId())
                .set(CommissionPaymentDO::getCustomerName, customerDOSaved.getName())
                .eq(CommissionPaymentDO::getCustomerId, customerDODeleted.getId()));
    }

    private void doMergeTargetLog(CustomerDO customerDOSaved, CustomerDO customerDODeleted) {
        targetLogService.update(new LambdaUpdateWrapper<TargetLogDO>()
                .set(TargetLogDO::getCustomerId, customerDOSaved.getId())
                .set(TargetLogDO::getCustomerType, customerDOSaved.getType())
                .eq(TargetLogDO::getCustomerId, customerDODeleted.getId()));
    }

    private void doMergeComissionPayable(CustomerDO customerDOSaved, CustomerDO customerDODeleted) {
        commissionPayableService.update(new LambdaUpdateWrapper<CommissionPayableDO>()
                .set(CommissionPayableDO::getCustomerId, customerDOSaved.getId())
                .set(CommissionPayableDO::getCustomerName, customerDOSaved.getName())
                .eq(CommissionPayableDO::getCustomerId, customerDODeleted.getId()));
    }

    //合并客户信用日志
    private void doMergeCustomerLevelLog(CustomerDO customerDOSaved,
                                         CustomerDO customerDODeleted,
                                         long loginUserId)
    {
        customerService.doMergeCustomerLevelLog(customerDOSaved.getId(),
                customerDODeleted.getId());
    }


    //合并产品品牌授权
    private void doMergeProductBrandEmpower(CustomerDO customerDOSaved,
                                            CustomerDO customerDODeleted,
                                            long loginUserId)
    {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<ProductBrandEmpowerDO>().
                        eq(ProductBrandEmpowerDO::getCustomerId,
                                customerDODeleted.getId());
        List<ProductBrandEmpowerDO> list =
                productBrandEmpowerService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (ProductBrandEmpowerDO d : list
        ) {
            d.setCustomerId(customerDOSaved.getId());
            productBrandEmpowerService.updateById(d);
        }
    }

    private void doMergeCustomerComplaint(CustomerDO customerDOSaved,
                                          CustomerDO customerDODeleted,
                                          long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerComplaintDO>().
                        eq(CustomerComplaintDO::getCustomerId,
                                customerDODeleted.getId());
        List<CustomerComplaintDO> list =
                customerComplaintService.selectList(q2);

        if (CollectionUtil.isEmpty(list)) return;

        for (CustomerComplaintDO d : list
        ) {
            d.setCustomerId(customerDOSaved.getId());
            customerComplaintService.updateById(d);
        }
    }

    //合并收款单
    private void doMergeeReceipt(CustomerDO customerDOSaved,
                                 CustomerDO customerDODeleted,
                                 long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<ReceiptDO>().
                        eq(ReceiptDO::getCustomerId, customerDODeleted.getId());
        List<ReceiptDO> list = receiptService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;
        for (ReceiptDO d : list
        ) {
            d.setCustomerId(customerDOSaved.getId());
            receiptService.updateById(d);
        }
    }


    //合并客户跟进
    private void doMergeCustomerFollowup(CustomerDO customerDOSaved,
                                         CustomerDO customerDODeleted,
                                         long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerFollowupDO>().
                        eq(CustomerFollowupDO::getCustomerId, customerDODeleted.getId());
        List<CustomerFollowupDO> list = customerFollowupService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (CustomerFollowupDO d : list
        ) {
            d.setCustomerId(customerDOSaved.getId());
            customerFollowupService.updateById(d);
        }
    }

    //合并报价单
    private void doMogeOffer(CustomerDO customerDOSaved,
                             CustomerDO customerDODeleted,
                             long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<OfferDO>().
                        eq(OfferDO::getRelationId, customerDODeleted.getId());
        List<OfferDO> list = offerService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (OfferDO o : list
        ) {
            o.setRelationId(customerDOSaved.getId());
            o.setUpdateTime(new Date());
            offerService.updateById(o);
        }
    }


    //合并联系人
    private void doMogeContact(CustomerDO customerDOSaved,
                               CustomerDO customerDODeleted,
                               long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerContactsDO>().
                        eq(CustomerContactsDO::getCustomerId, customerDODeleted.getId());
        List<CustomerContactsDO> cuscontactList2 =
                customerContactsService.selectList(q2);
        if (CollectionUtil.isNotEmpty(cuscontactList2)) {
            for (CustomerContactsDO d : cuscontactList2
            ) {
                try {
                    // 1、联系人copy一份
                    CustomerContactsDO newCustomer = new CustomerContactsDO();
                    BeanUtil.copyProperties(d, newCustomer);
                    newCustomer.setId(null);
                    //被合并的客户联系人，默认为非默认
                    newCustomer.setIsDefault(0);
                    // 设置被合并客户的联系人
                    newCustomer.setCustomerId(customerDOSaved.getId());
                    customerContactsService.save(newCustomer);

                    // 2、删除被合并客户的联系人
                    customerContactsService.deleteCustomerContacts(d.getId());
                } catch (Exception e) {
                    log.error("合并客户联系人失败", e);
                }
            }
        }
    }
}
